@@ -20,6 +20,7 @@ pub use aws_credential_types::provider::SharedCredentialsProvider;
 use aws_smithy_async::rt::sleep::AsyncSleep;
 pub use aws_smithy_async::rt::sleep::SharedAsyncSleep;
 pub use aws_smithy_async::time::{SharedTimeSource, TimeSource};
+use aws_smithy_runtime_api::client::auth::AuthSchemePreference;
 use aws_smithy_runtime_api::client::behavior_version::BehaviorVersion;
 use aws_smithy_runtime_api::client::http::HttpClient;
 pub use aws_smithy_runtime_api::client::http::SharedHttpClient;
@@ -81,6 +82,17 @@ See the developer guide on [account-based endpoints](https://docs.aws.amazon.com
 for more information.
 
 For services that do not use the account-based endpoints, this setting does nothing.
+" };
+        (auth_scheme_preference) => {
+"Set the auth scheme preference for an auth scheme resolver
+(typically the default auth scheme resolver).
+
+Each operation has a predefined order of auth schemes, as determined by the service,
+for auth scheme resolution. By using the auth scheme preference, customers
+can reorder the schemes resolved by the auth scheme resolver.
+
+The preference list is intended as a hint rather than a strict override.
+Any schemes not present in the originally resolved auth schemes will be ignored.
 " };
     }
 }
@@ -89,6 +101,7 @@ For services that do not use the account-based endpoints, this setting does noth
 #[derive(Debug, Clone)]
 pub struct SdkConfig {
     app_name: Option<AppName>,
+    auth_scheme_preference: Option<AuthSchemePreference>,
     identity_cache: Option<SharedIdentityCache>,
     credentials_provider: Option<SharedCredentialsProvider>,
     token_provider: Option<SharedTokenProvider>,
@@ -120,6 +133,7 @@ pub struct SdkConfig {
 #[derive(Debug, Default)]
 pub struct Builder {
     app_name: Option<AppName>,
+    auth_scheme_preference: Option<AuthSchemePreference>,
     identity_cache: Option<SharedIdentityCache>,
     credentials_provider: Option<SharedCredentialsProvider>,
     token_provider: Option<SharedTokenProvider>,
@@ -777,6 +791,24 @@ impl Builder {
         self
     }
 
+    #[doc = docs_for!(auth_scheme_preference)]
+    pub fn auth_scheme_preference(
+        mut self,
+        auth_scheme_preference: impl Into<AuthSchemePreference>,
+    ) -> Self {
+        self.set_auth_scheme_preference(Some(auth_scheme_preference));
+        self
+    }
+
+    #[doc = docs_for!(auth_scheme_preference)]
+    pub fn set_auth_scheme_preference(
+        &mut self,
+        auth_scheme_preference: Option<impl Into<AuthSchemePreference>>,
+    ) -> &mut Self {
+        self.auth_scheme_preference = auth_scheme_preference.map(|pref| pref.into());
+        self
+    }
+
     /// Set the origin of a setting.
     ///
     /// This is used internally to understand how to merge config structs while
@@ -789,6 +821,7 @@ impl Builder {
     pub fn build(self) -> SdkConfig {
         SdkConfig {
             app_name: self.app_name,
+            auth_scheme_preference: self.auth_scheme_preference,
             identity_cache: self.identity_cache,
             credentials_provider: self.credentials_provider,
             token_provider: self.token_provider,
@@ -896,6 +929,11 @@ impl SdkConfig {
         self.account_id_endpoint_mode.as_ref()
     }
 
+    /// Configured auth scheme preference
+    pub fn auth_scheme_preference(&self) -> Option<&AuthSchemePreference> {
+        self.auth_scheme_preference.as_ref()
+    }
+
     /// Configured endpoint URL
     pub fn endpoint_url(&self) -> Option<&str> {
         self.endpoint_url.as_deref()
@@ -1020,6 +1058,7 @@ impl SdkConfig {
     pub fn into_builder(self) -> Builder {
         Builder {
             app_name: self.app_name,
+            auth_scheme_preference: self.auth_scheme_preference,
             identity_cache: self.identity_cache,
             credentials_provider: self.credentials_provider,
             token_provider: self.token_provider,
