@@ -30,11 +30,8 @@
 //! module allow you to retrieve these access tokens, and can be used with
 //! the Google Cloud client libraries for Rust.
 //!
-//! While the Google Cloud client libraries for Rust default to
-//! using the types defined in this module. You may want to use said types directly
-//! to customize some of the properties of these credentials.
+//! ## Example: Creating credentials with a custom quota project
 //!
-//! # Example
 //! ```
 //! # use google_cloud_auth::credentials::mds::Builder;
 //! # use google_cloud_auth::credentials::Credentials;
@@ -49,6 +46,28 @@
 //! # });
 //! ```
 //!
+//! ## Example: Creating credentials with custom retry behavior
+//!
+//! ```
+//! # use google_cloud_auth::credentials::mds::Builder;
+//! # use google_cloud_auth::credentials::Credentials;
+//! # use http::Extensions;
+//! # use std::time::Duration;
+//! # tokio_test::block_on(async {
+//! use gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
+//! use gax::exponential_backoff::ExponentialBackoff;
+//! let backoff = ExponentialBackoff::default();
+//! let credentials: Credentials = Builder::default()
+//!     .with_retry_policy(AlwaysRetry.with_attempt_limit(3))
+//!     .with_backoff_policy(backoff)
+//!     .build()?;
+//! let headers = credentials.headers(Extensions::new()).await?;
+//! println!("Headers: {headers:?}");
+//! # Ok::<(), anyhow::Error>(())
+//! # });
+//! ```
+//!
+//! [Application Default Credentials]: https://cloud.google.com/docs/authentication/application-default-credentials
 //! [Cloud Run]: https://cloud.google.com/run
 //! [default service account]: https://cloud.google.com/iam/docs/service-account-types#default
 //! [gce-link]: https://cloud.google.com/products/compute
@@ -59,11 +78,15 @@ use crate::credentials::dynamic::CredentialsProvider;
 use crate::credentials::{CacheableResource, Credentials, DEFAULT_UNIVERSE_DOMAIN};
 use crate::errors::CredentialsError;
 use crate::headers_util::build_cacheable_headers;
+use crate::retry::{Builder as RetryTokenProviderBuilder, TokenProviderWithRetry};
 use crate::token::{CachedTokenProvider, Token, TokenProvider};
 use crate::token_cache::TokenCache;
 use crate::{BuildResult, Result};
 use async_trait::async_trait;
 use bon::Builder;
+use gax::backoff_policy::BackoffPolicyArg;
+use gax::retry_policy::RetryPolicyArg;
+use gax::retry_throttler::RetryThrottlerArg;
 use http::{Extensions, HeaderMap, HeaderValue};
 use reqwest::Client;
 use std::default::Default;
@@ -113,6 +136,7 @@ pub struct Builder {
     scopes: Option<Vec<String>>,
     universe_domain: Option<String>,
     created_by_adc: bool,
+    retry_builder: RetryTokenProviderBuilder,
 }
 
 impl Builder {
@@ -176,6 +200,70 @@ impl Builder {
         self
     }
 
+    /// Configure the retry policy for fetching tokens.
+    ///
+    /// The retry policy controls how to handle retries, and sets limits on
+    /// the number of attempts or the total time spent retrying.
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::mds::Builder;
+    /// # tokio_test::block_on(async {
+    /// use gax::retry_policy::{AlwaysRetry, RetryPolicyExt};
+    /// let credentials = Builder::default()
+    ///     .with_retry_policy(AlwaysRetry.with_attempt_limit(3))
+    ///     .build();
+    /// # });
+    /// ```
+    pub fn with_retry_policy<V: Into<RetryPolicyArg>>(mut self, v: V) -> Self {
+        self.retry_builder = self.retry_builder.with_retry_policy(v.into());
+        self
+    }
+
+    /// Configure the retry backoff policy.
+    ///
+    /// The backoff policy controls how long to wait in between retry attempts.
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::mds::Builder;
+    /// # use std::time::Duration;
+    /// # tokio_test::block_on(async {
+    /// use gax::exponential_backoff::ExponentialBackoff;
+    /// let policy = ExponentialBackoff::default();
+    /// let credentials = Builder::default()
+    ///     .with_backoff_policy(policy)
+    ///     .build();
+    /// # });
+    /// ```
+    pub fn with_backoff_policy<V: Into<BackoffPolicyArg>>(mut self, v: V) -> Self {
+        self.retry_builder = self.retry_builder.with_backoff_policy(v.into());
+        self
+    }
+
+    /// Configure the retry throttler.
+    ///
+    /// Advanced applications may want to configure a retry throttler to
+    /// [Address Cascading Failures] and when [Handling Overload] conditions.
+    /// The authentication library throttles its retry loop, using a policy to
+    /// control the throttling algorithm. Use this method to fine tune or
+    /// customize the default retry throttler.
+    ///
+    /// [Handling Overload]: https://sre.google/sre-book/handling-overload/
+    /// [Address Cascading Failures]: https://sre.google/sre-book/addressing-cascading-failures/
+    ///
+    /// ```
+    /// # use google_cloud_auth::credentials::mds::Builder;
+    /// # tokio_test::block_on(async {
+    /// use gax::retry_throttler::AdaptiveThrottler;
+    /// let credentials = Builder::default()
+    ///     .with_retry_throttler(AdaptiveThrottler::default())
+    ///     .build();
+    /// # });
+    /// ```
+    pub fn with_retry_throttler<V: Into<RetryThrottlerArg>>(mut self, v: V) -> Self {
+        self.retry_builder = self.retry_builder.with_retry_throttler(v.into());
+        self
+    }
+
     // This method is used to build mds credentials from ADC
     pub(crate) fn from_adc() -> Self {
         Self {
@@ -184,14 +272,14 @@ impl Builder {
         }
     }
 
-    fn build_token_provider(self) -> MDSAccessTokenProvider {
+    fn build_token_provider(self) -> TokenProviderWithRetry<MDSAccessTokenProvider> {
         let final_endpoint: String;
         let endpoint_overridden: bool;
 
         // Determine the endpoint and whether it was overridden
         if let Ok(host_from_env) = std::env::var(GCE_METADATA_HOST_ENV_VAR) {
             // Check GCE_METADATA_HOST environment variable first
-            final_endpoint = format!("http://{}", host_from_env);
+            final_endpoint = format!("http://{host_from_env}");
             endpoint_overridden = true;
         } else if let Some(builder_endpoint) = self.endpoint {
             // Else, check if an endpoint was provided to the mds::Builder
@@ -203,12 +291,13 @@ impl Builder {
             endpoint_overridden = false;
         };
 
-        MDSAccessTokenProvider::builder()
+        let tp = MDSAccessTokenProvider::builder()
             .endpoint(final_endpoint)
             .maybe_scopes(self.scopes)
             .endpoint_overridden(endpoint_overridden)
             .created_by_adc(self.created_by_adc)
-            .build()
+            .build();
+        self.retry_builder.build(tp)
     }
 
     /// Returns a [Credentials] instance with the configured settings.
@@ -242,13 +331,6 @@ where
     }
 }
 
-#[derive(Clone, Debug, PartialEq, serde::Deserialize, serde::Serialize)]
-struct ServiceAccountInfo {
-    email: String,
-    scopes: Option<Vec<String>>,
-    aliases: Option<Vec<String>>,
-}
-
 #[derive(Clone, Debug, PartialEq, serde::Deserialize, serde::Serialize)]
 struct MDSTokenResponse {
     access_token: String,
@@ -337,46 +419,120 @@ impl TokenProvider for MDSAccessTokenProvider {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use crate::credentials::QUOTA_PROJECT_KEY;
-    use crate::credentials::test::{
-        get_headers_from_cache, get_token_from_headers, get_token_type_from_headers,
+    use crate::credentials::tests::{
+        find_source_error, get_headers_from_cache, get_mock_auth_retry_policy,
+        get_mock_backoff_policy, get_mock_retry_throttler, get_token_from_headers,
+        get_token_type_from_headers,
     };
     use crate::errors;
-    use crate::token::test::MockTokenProvider;
-    use axum::extract::Query;
-    use axum::response::IntoResponse;
+    use crate::errors::CredentialsError;
+    use crate::token::tests::MockTokenProvider;
+    use http::HeaderValue;
     use http::header::AUTHORIZATION;
+    use httptest::cycle;
+    use httptest::matchers::{all_of, contains, request, url_decoded};
+    use httptest::responders::{json_encoded, status_code};
+    use httptest::{Expectation, Server};
     use reqwest::StatusCode;
-    use reqwest::header::HeaderMap;
     use scoped_env::ScopedEnv;
-    use serde::Deserialize;
-    use serde_json::Value;
     use serial_test::{parallel, serial};
-    use std::collections::HashMap;
     use std::error::Error;
-    use std::sync::Mutex;
     use test_case::test_case;
-    use tokio::task::JoinHandle;
     use url::Url;
 
-    type TestResult = std::result::Result<(), Box<dyn std::error::Error>>;
+    type TestResult = anyhow::Result<()>;
 
-    // Define a struct to capture query parameters
-    #[derive(Debug, Clone, Deserialize, PartialEq)]
-    struct TokenQueryParams {
-        scopes: Option<String>,
-        recursive: Option<String>,
+    #[tokio::test]
+    #[parallel]
+    async fn test_mds_retries_on_transient_failures() -> TestResult {
+        let mut server = Server::run();
+        server.expect(
+            Expectation::matching(request::path(format!("{MDS_DEFAULT_URI}/token")))
+                .times(3)
+                .respond_with(status_code(503)),
+        );
+
+        let provider = Builder::default()
+            .with_endpoint(format!("http://{}", server.addr()))
+            .with_retry_policy(get_mock_auth_retry_policy(3))
+            .with_backoff_policy(get_mock_backoff_policy())
+            .with_retry_throttler(get_mock_retry_throttler())
+            .build_token_provider();
+
+        let err = provider.token().await.unwrap_err();
+        assert!(!err.is_transient());
+        server.verify_and_clear();
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
+    async fn test_mds_does_not_retry_on_non_transient_failures() -> TestResult {
+        let mut server = Server::run();
+        server.expect(
+            Expectation::matching(request::path(format!("{MDS_DEFAULT_URI}/token")))
+                .times(1)
+                .respond_with(status_code(401)),
+        );
+
+        let provider = Builder::default()
+            .with_endpoint(format!("http://{}", server.addr()))
+            .with_retry_policy(get_mock_auth_retry_policy(1))
+            .with_backoff_policy(get_mock_backoff_policy())
+            .with_retry_throttler(get_mock_retry_throttler())
+            .build_token_provider();
+
+        let err = provider.token().await.unwrap_err();
+        assert!(!err.is_transient());
+        server.verify_and_clear();
+        Ok(())
+    }
+
+    #[tokio::test]
+    #[parallel]
+    async fn test_mds_retries_for_success() -> TestResult {
+        let mut server = Server::run();
+        let response = MDSTokenResponse {
+            access_token: "test-access-token".to_string(),
+            expires_in: Some(3600),
+            token_type: "test-token-type".to_string(),
+        };
+
+        server.expect(
+            Expectation::matching(request::path(format!("{MDS_DEFAULT_URI}/token")))
+                .times(3)
+                .respond_with(cycle![
+                    status_code(503).body("try-again"),
+                    status_code(503).body("try-again"),
+                    status_code(200)
+                        .append_header("Content-Type", "application/json")
+                        .body(serde_json::to_string(&response).unwrap()),
+                ]),
+        );
+
+        let provider = Builder::default()
+            .with_endpoint(format!("http://{}", server.addr()))
+            .with_retry_policy(get_mock_auth_retry_policy(3))
+            .with_backoff_policy(get_mock_backoff_policy())
+            .with_retry_throttler(get_mock_retry_throttler())
+            .build_token_provider();
+
+        let token = provider.token().await?;
+        assert_eq!(token.token, "test-access-token");
+
+        server.verify_and_clear();
+        Ok(())
     }
 
     #[test]
     fn validate_default_endpoint_urls() {
-        let default_endpoint_address = Url::parse(&format!("{}{}", METADATA_ROOT, MDS_DEFAULT_URI));
+        let default_endpoint_address = Url::parse(&format!("{METADATA_ROOT}{MDS_DEFAULT_URI}"));
         assert!(default_endpoint_address.is_ok());
 
-        let token_endpoint_address =
-            Url::parse(&format!("{}{}/token", METADATA_ROOT, MDS_DEFAULT_URI));
+        let token_endpoint_address = Url::parse(&format!("{METADATA_ROOT}{MDS_DEFAULT_URI}/token"));
         assert!(token_endpoint_address.is_ok());
     }
 
@@ -466,21 +622,16 @@ mod test {
     #[tokio::test]
     #[serial]
     async fn adc_no_mds() -> TestResult {
-        let err = Builder::from_adc()
-            .build_token_provider()
-            .token()
-            .await
-            .unwrap_err();
+        let Err(err) = Builder::from_adc().build_token_provider().token().await else {
+            // The environment has an MDS, skip the test.
+            return Ok(());
+        };
 
-        assert!(err.is_transient(), "{err:?}");
+        let original_err = find_source_error::<CredentialsError>(&err).unwrap();
         assert!(
-            err.to_string().contains("application-default"),
+            original_err.to_string().contains("application-default"),
             "display={err}, debug={err:?}"
         );
-        let source = err
-            .source()
-            .and_then(|e| e.downcast_ref::<reqwest::Error>());
-        assert!(matches!(source, Some(e) if e.status().is_none()), "{err:?}");
 
         Ok(())
     }
@@ -498,14 +649,13 @@ mod test {
 
         let _e = ScopedEnv::remove(super::GCE_METADATA_HOST_ENV_VAR);
 
-        assert!(err.is_transient(), "{err:?}");
+        let original_err = find_source_error::<CredentialsError>(&err).unwrap();
+        assert!(original_err.is_transient());
         assert!(
-            !err.to_string().contains("application-default"),
+            !original_err.to_string().contains("application-default"),
             "display={err}, debug={err:?}"
         );
-        let source = err
-            .source()
-            .and_then(|e| e.downcast_ref::<reqwest::Error>());
+        let source = find_source_error::<reqwest::Error>(&err);
         assert!(matches!(source, Some(e) if e.status().is_none()), "{err:?}");
 
         Ok(())
@@ -514,90 +664,40 @@ mod test {
     #[tokio::test]
     #[serial]
     async fn builder_no_mds() -> TestResult {
-        let e = Builder::default()
-            .build_token_provider()
-            .token()
-            .await
-            .err()
-            .unwrap();
+        let Err(e) = Builder::default().build_token_provider().token().await else {
+            // The environment has an MDS, skip the test.
+            return Ok(());
+        };
 
-        assert!(e.is_transient(), "{e:?}");
+        let original_err = find_source_error::<CredentialsError>(&e).unwrap();
         assert!(
-            !format!("{:?}", e.source()).contains("application-default"),
+            !format!("{:?}", original_err.source()).contains("application-default"),
             "{e:?}"
         );
 
         Ok(())
     }
 
-    fn handle_token_factory(
-        response_code: StatusCode,
-        response_headers: HeaderMap,
-        response_body: Value,
-    ) -> impl IntoResponse {
-        (response_code, response_headers, response_body.to_string()).into_response()
-    }
-
-    type Handlers = HashMap<String, (StatusCode, Value, TokenQueryParams, Arc<Mutex<i32>>)>;
-
-    // Starts a server running locally that responds on multiple paths.
-    // Returns an (endpoint, server) pair.
-    async fn start(path_handlers: Handlers) -> (String, JoinHandle<()>) {
-        let mut app = axum::Router::new();
-
-        for (path, (code, body, expected_query, call_count)) in path_handlers {
-            let header_map = HeaderMap::new();
-            let handler = move |Query(query): Query<TokenQueryParams>| {
-                let body = body.clone();
-                let header_map = header_map.clone();
-                async move {
-                    assert_eq!(expected_query, query);
-                    let mut count = call_count.lock().unwrap();
-                    *count += 1;
-                    handle_token_factory(code, header_map, body)
-                }
-            };
-            app = app.route(&path, axum::routing::get(handler));
-        }
-
-        let listener = tokio::net::TcpListener::bind("127.0.0.1:0").await.unwrap();
-        let addr = listener.local_addr().unwrap();
-        let server = tokio::spawn(async move {
-            axum::serve(listener, app).await.unwrap();
-        });
-        (format!("http://{}:{}", addr.ip(), addr.port()), server)
-    }
-
     #[tokio::test]
     #[serial]
-    async fn test_gce_metadata_host_env_var() {
-        let scopes = ["scope1".to_string(), "scope2".to_string()];
+    async fn test_gce_metadata_host_env_var() -> TestResult {
+        let server = Server::run();
+        let scopes = ["scope1", "scope2"];
         let response = MDSTokenResponse {
             access_token: "test-access-token".to_string(),
             expires_in: Some(3600),
             token_type: "test-token-type".to_string(),
         };
-        let response_body = serde_json::to_value(&response).unwrap();
-
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                response_body,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
-
-        // Trim out 'http://' from the endpoint provided by the fake server
-        let _e = ScopedEnv::set(
-            super::GCE_METADATA_HOST_ENV_VAR,
-            endpoint.strip_prefix("http://").unwrap_or(&endpoint),
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(json_encoded(response)),
         );
+
+        let addr = server.addr().to_string();
+        let _e = ScopedEnv::set(super::GCE_METADATA_HOST_ENV_VAR, &addr);
         let mdsc = Builder::default()
             .with_scopes(["scope1", "scope2"])
             .build()
@@ -609,36 +709,30 @@ mod test {
             get_token_from_headers(headers).unwrap(),
             "test-access-token"
         );
+        Ok(())
     }
 
     #[tokio::test]
     #[parallel]
     async fn headers_success_with_quota_project() -> TestResult {
-        let scopes = ["scope1".to_string(), "scope2".to_string()];
+        let server = Server::run();
+        let scopes = ["scope1", "scope2"];
         let response = MDSTokenResponse {
             access_token: "test-access-token".to_string(),
             expires_in: Some(3600),
             token_type: "test-token-type".to_string(),
         };
-        let response_body = serde_json::to_value(&response).unwrap();
-
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                response_body,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(json_encoded(response)),
+        );
 
         let mdsc = Builder::default()
             .with_scopes(["scope1", "scope2"])
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .with_quota_project_id("test-project")
             .build()?;
 
@@ -661,32 +755,25 @@ mod test {
     #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
     #[parallel]
     async fn token_caching() -> TestResult {
+        let mut server = Server::run();
         let scopes = vec!["scope1".to_string()];
         let response = MDSTokenResponse {
             access_token: "test-access-token".to_string(),
             expires_in: Some(3600),
             token_type: "test-token-type".to_string(),
         };
-        let response_body = serde_json::to_value(&response).unwrap();
-
-        let call_count = Arc::new(Mutex::new(0));
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                response_body,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                call_count.clone(),
-            ),
-        )]))
-        .await;
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .times(1)
+            .respond_with(json_encoded(response)),
+        );
 
         let mdsc = Builder::default()
             .with_scopes(scopes)
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .build()?;
         let headers = mdsc.headers(Extensions::new()).await?;
         assert_eq!(
@@ -700,7 +787,7 @@ mod test {
         );
 
         // validate that the inner token provider is called only once
-        assert_eq!(*call_count.lock().unwrap(), 1);
+        server.verify_and_clear();
 
         Ok(())
     }
@@ -708,31 +795,23 @@ mod test {
     #[tokio::test(start_paused = true)]
     #[parallel]
     async fn token_provider_full() -> TestResult {
+        let server = Server::run();
         let scopes = vec!["scope1".to_string()];
         let response = MDSTokenResponse {
             access_token: "test-access-token".to_string(),
             expires_in: Some(3600),
             token_type: "test-token-type".to_string(),
         };
-        let response_body = serde_json::to_value(&response).unwrap();
-
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                response_body,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
-        println!("endpoint = {endpoint}");
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(json_encoded(response)),
+        );
 
         let token = Builder::default()
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .with_scopes(scopes)
             .build_token_provider()
             .token()
@@ -753,29 +832,19 @@ mod test {
     #[tokio::test(start_paused = true)]
     #[parallel]
     async fn token_provider_full_no_scopes() -> TestResult {
+        let server = Server::run();
         let response = MDSTokenResponse {
             access_token: "test-access-token".to_string(),
             expires_in: Some(3600),
             token_type: "test-token-type".to_string(),
         };
-        let response_body = serde_json::to_value(&response).unwrap();
+        server.expect(
+            Expectation::matching(request::path(format!("{MDS_DEFAULT_URI}/token")))
+                .respond_with(json_encoded(response)),
+        );
 
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                response_body,
-                TokenQueryParams {
-                    scopes: None,
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
-        println!("endpoint = {endpoint}");
         let token = Builder::default()
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .build_token_provider()
             .token()
             .await?;
@@ -795,30 +864,23 @@ mod test {
     #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
     #[parallel]
     async fn credential_provider_full() -> TestResult {
+        let server = Server::run();
         let scopes = vec!["scope1".to_string()];
         let response = MDSTokenResponse {
             access_token: "test-access-token".to_string(),
             expires_in: None,
             token_type: "test-token-type".to_string(),
         };
-        let response_body = serde_json::to_value(&response).unwrap();
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                response_body,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
-        println!("endpoint = {endpoint}");
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(json_encoded(response)),
+        );
 
         let mdsc = Builder::default()
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .with_scopes(scopes)
             .build()?;
         let headers = mdsc.headers(Extensions::new()).await?;
@@ -837,31 +899,24 @@ mod test {
     #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
     #[parallel]
     async fn credentials_headers_retryable_error() -> TestResult {
+        let server = Server::run();
         let scopes = vec!["scope1".to_string()];
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::SERVICE_UNAVAILABLE,
-                serde_json::to_value("try again")?,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(status_code(503)),
+        );
 
         let mdsc = Builder::default()
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .with_scopes(scopes)
             .build()?;
         let err = mdsc.headers(Extensions::new()).await.unwrap_err();
-        assert!(err.is_transient());
-        assert!(err.to_string().contains("try again"), "{err:?}");
-        let source = err
-            .source()
-            .and_then(|e| e.downcast_ref::<reqwest::Error>());
+        let original_err = find_source_error::<CredentialsError>(&err).unwrap();
+        assert!(original_err.is_transient());
+        let source = find_source_error::<reqwest::Error>(&err);
         assert!(
             matches!(source, Some(e) if e.status() == Some(StatusCode::SERVICE_UNAVAILABLE)),
             "{err:?}"
@@ -873,32 +928,25 @@ mod test {
     #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
     #[parallel]
     async fn credentials_headers_nonretryable_error() -> TestResult {
+        let server = Server::run();
         let scopes = vec!["scope1".to_string()];
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::UNAUTHORIZED,
-                serde_json::to_value("epic fail".to_string())?,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(status_code(401)),
+        );
 
         let mdsc = Builder::default()
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .with_scopes(scopes)
             .build()?;
 
         let err = mdsc.headers(Extensions::new()).await.unwrap_err();
-        assert!(!err.is_transient());
-        assert!(err.to_string().contains("epic fail"), "{err:?}");
-        let source = err
-            .source()
-            .and_then(|e| e.downcast_ref::<reqwest::Error>());
+        let original_err = find_source_error::<CredentialsError>(&err).unwrap();
+        assert!(!original_err.is_transient());
+        let source = find_source_error::<reqwest::Error>(&err);
         assert!(
             matches!(source, Some(e) if e.status() == Some(StatusCode::UNAUTHORIZED)),
             "{err:?}"
@@ -910,23 +958,18 @@ mod test {
     #[tokio::test(flavor = "multi_thread", worker_threads = 2)]
     #[parallel]
     async fn credentials_headers_malformed_response_is_nonretryable() -> TestResult {
+        let server = Server::run();
         let scopes = vec!["scope1".to_string()];
-        let (endpoint, _server) = start(Handlers::from([(
-            format!("{}/token", MDS_DEFAULT_URI),
-            (
-                StatusCode::OK,
-                serde_json::to_value("bad json".to_string())?,
-                TokenQueryParams {
-                    scopes: Some(scopes.join(",")),
-                    recursive: None,
-                },
-                Arc::new(Mutex::new(0)),
-            ),
-        )]))
-        .await;
+        server.expect(
+            Expectation::matching(all_of![
+                request::path(format!("{MDS_DEFAULT_URI}/token")),
+                request::query(url_decoded(contains(("scopes", scopes.join(",")))))
+            ])
+            .respond_with(json_encoded("bad json")),
+        );
 
         let mdsc = Builder::default()
-            .with_endpoint(endpoint)
+            .with_endpoint(format!("http://{}", server.addr()))
             .with_scopes(scopes)
             .build()?;
 
