@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use crate::retry_state::RetryState;
 use crate::throttle_result::ThrottleResult;
 
 use super::Result;
@@ -48,7 +49,7 @@ impl RetryLoopAttempt {
 /// In between calls the function waits the amount of time prescribed by the
 /// backoff policy, using `sleep` to implement any sleep.
 pub async fn retry_loop<F, S, Response>(
-    inner: F,
+    mut inner: F,
     sleep: S,
     idempotent: bool,
     retry_throttler: Arc<Mutex<dyn RetryThrottler>>,
@@ -56,14 +57,17 @@ pub async fn retry_loop<F, S, Response>(
     backoff_policy: Arc<dyn BackoffPolicy>,
 ) -> Result<Response>
 where
-    F: AsyncFn(Option<Duration>) -> Result<Response> + Send,
+    F: AsyncFnMut(Option<Duration>) -> Result<Response> + Send,
     S: AsyncFn(Duration) -> () + Send,
 {
     let loop_start = tokio::time::Instant::now().into_std();
     let mut attempt = RetryLoopAttempt::Initial;
     loop {
         let mut attempt_count = attempt.count();
-        let remaining_time = retry_policy.remaining_time(loop_start, attempt_count);
+        let state = RetryState::new(idempotent)
+            .set_attempt_count(attempt_count)
+            .set_start(loop_start);
+        let remaining_time = retry_policy.remaining_time(&state);
 
         if let RetryLoopAttempt::Retry(attempt_count, delay, prev_error) = attempt {
             if remaining_time.is_some_and(|remaining| remaining < delay) {
@@ -77,7 +81,7 @@ where
                 .throttle_retry_attempt()
             {
                 // This counts as an error for the purposes of the retry policy.
-                let error = match retry_policy.on_throttle(loop_start, attempt_count, prev_error) {
+                let error = match retry_policy.on_throttle(&state, prev_error) {
                     ThrottleResult::Exhausted(e) => {
                         return Err(e);
                     }
@@ -89,6 +93,7 @@ where
             }
         }
         attempt_count += 1;
+        let state = state.set_attempt_count(attempt_count);
         match inner(remaining_time).await {
             Ok(r) => {
                 retry_throttler
@@ -98,7 +103,7 @@ where
                 return Ok(r);
             }
             Err(e) => {
-                let flow = retry_policy.on_error(loop_start, attempt_count, idempotent, e);
+                let flow = retry_policy.on_error(&state, e);
                 let delay = backoff_policy.on_failure(loop_start, attempt_count);
                 retry_throttler
                     .lock()
@@ -131,9 +136,9 @@ pub fn effective_timeout(
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
-    use crate::error::{Error, rpc::Code, rpc::Status};
+    use crate::error::{Error, rpc::Code, rpc::Status, rpc::StatusDetails};
     use std::error::Error as _;
     use test_case::test_case;
 
@@ -214,7 +219,7 @@ mod test {
         retry_policy
             .expect_on_error()
             .once()
-            .returning(|_, _, _, e| RetryResult::Permanent(e));
+            .returning(|_, e| RetryResult::Permanent(e));
         let mut backoff_policy = MockBackoffPolicy::new();
         backoff_policy
             .expect_on_failure()
@@ -311,8 +316,8 @@ mod test {
         retry_policy
             .expect_on_error()
             .times(2)
-            .withf(move |_, _, idempotent, _| idempotent == &expected_idempotency)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .withf(move |state, _| state.idempotent == expected_idempotency)
+            .returning(|_, e| RetryResult::Continue(e));
 
         let mut backoff_seq = mockall::Sequence::new();
         let mut backoff_policy = MockBackoffPolicy::new();
@@ -351,15 +356,15 @@ mod test {
     async fn too_many_transients() -> anyhow::Result<()> {
         // This test simulates a server responding with two transient errors
         // and the retry policy stops after the second attempt.
-        const ERRORS: usize = 2;
+        const ERRORS: usize = 3;
         let mut call_seq = mockall::Sequence::new();
         let mut call = MockCall::new();
-        for _ in 0..ERRORS {
+        for i in 0..ERRORS {
             call.expect_call()
                 .once()
                 .withf(|d| d.is_none())
                 .in_sequence(&mut call_seq)
-                .returning(|_| transient());
+                .returning(move |_| numbered_transient(i));
         }
         let inner = async move |d| call.call(d);
 
@@ -393,12 +398,12 @@ mod test {
             .expect_on_error()
             .times(ERRORS - 1)
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .returning(|_, e| RetryResult::Continue(e));
         retry_policy
             .expect_on_error()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, _, e| RetryResult::Exhausted(e));
+            .returning(|_, e| RetryResult::Exhausted(e));
         let mut backoff_policy = MockBackoffPolicy::new();
         backoff_policy
             .expect_on_failure()
@@ -421,7 +426,13 @@ mod test {
             to_backoff_policy(backoff_policy),
         )
         .await;
-        assert!(response.is_err(), "{response:?}");
+        let err = response.unwrap_err();
+        let status = err.status().unwrap();
+        let detail = status.details.first().unwrap();
+        assert!(
+            matches!(detail, StatusDetails::DebugInfo(e) if e.detail == format!("count={}", ERRORS - 1) ),
+            "{status:?}"
+        );
         Ok(())
     }
 
@@ -470,12 +481,12 @@ mod test {
             .expect_on_error()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .returning(|_, e| RetryResult::Continue(e));
         retry_policy
             .expect_on_error()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, _, e| RetryResult::Permanent(e));
+            .returning(|_, e| RetryResult::Permanent(e));
         let mut backoff_policy = MockBackoffPolicy::new();
         backoff_policy
             .expect_on_failure()
@@ -552,12 +563,12 @@ mod test {
             .expect_on_error()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .returning(|_, e| RetryResult::Continue(e));
         retry_policy
             .expect_on_throttle()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, e| ThrottleResult::Continue(e));
+            .returning(|_, e| ThrottleResult::Continue(e));
 
         let mut backoff_policy = MockBackoffPolicy::new();
         backoff_policy
@@ -618,12 +629,12 @@ mod test {
             .expect_on_error()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .returning(|_, e| RetryResult::Continue(e));
         retry_policy
             .expect_on_throttle()
             .once()
             .in_sequence(&mut retry_seq)
-            .returning(|_, _, e| ThrottleResult::Exhausted(e));
+            .returning(|_, e| ThrottleResult::Exhausted(e));
 
         let mut backoff_policy = MockBackoffPolicy::new();
         backoff_policy
@@ -685,7 +696,7 @@ mod test {
             .expect_on_error()
             .once()
             .in_sequence(&mut seq)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .returning(|_, e| RetryResult::Continue(e));
 
         // The backoff policy wants to sleep for longer than the overall timeout.
         backoff_policy
@@ -767,7 +778,7 @@ mod test {
             .expect_on_error()
             .once()
             .in_sequence(&mut seq)
-            .returning(|_, _, _, e| RetryResult::Continue(e));
+            .returning(|_, e| RetryResult::Continue(e));
 
         // The backoff policy returns an instantaneous sleep.
         backoff_policy
@@ -812,7 +823,7 @@ mod test {
             .expect_on_throttle()
             .once()
             .in_sequence(&mut seq)
-            .returning(|_, _, e| ThrottleResult::Continue(e));
+            .returning(|_, e| ThrottleResult::Continue(e));
 
         // The backoff policy wants to sleep for longer than the overall timeout.
         backoff_policy
@@ -868,6 +879,12 @@ mod test {
         Err(Error::service(transient_status()))
     }
 
+    fn numbered_transient(i: usize) -> Result<String> {
+        Err(Error::service(transient_status().set_details([
+            StatusDetails::DebugInfo(rpc::model::DebugInfo::new().set_detail(format!("count={i}"))),
+        ])))
+    }
+
     fn permanent() -> Result<String> {
         let status = Status::default()
             .set_code(Code::PermissionDenied)
@@ -913,9 +930,9 @@ mod test {
         #[derive(Debug)]
         RetryPolicy {}
         impl RetryPolicy for RetryPolicy {
-            fn on_error(&self, loop_start: std::time::Instant, attempt_count: u32, idempotent: bool, error: Error) -> RetryResult;
-            fn on_throttle(&self, loop_start: std::time::Instant, attempt_count: u32, error: Error) -> ThrottleResult;
-            fn remaining_time(&self, loop_start: std::time::Instant, attempt_count: u32) -> Option<Duration>;
+            fn on_error(&self,       state: &RetryState, error: Error) -> RetryResult;
+            fn on_throttle(&self,    state: &RetryState, error: Error) -> ThrottleResult;
+            fn remaining_time(&self, state: &RetryState) -> Option<Duration>;
         }
         impl std::clone::Clone for RetryPolicy {
             fn clone(&self) -> Self;
