@@ -121,7 +121,6 @@ macro_rules! impl_message {
                 concat!("type.googleapis.com/google.protobuf.", stringify!($t))
             }
 
-            #[allow(private_interfaces)]
             fn serializer() -> impl crate::message::MessageSerializer<Self> {
                 crate::message::ValueSerializer::<Self>::new()
             }
@@ -163,7 +162,6 @@ impl crate::message::Message for UInt64Value {
         "type.googleapis.com/google.protobuf.UInt64Value"
     }
 
-    #[allow(private_interfaces)]
     fn serializer() -> impl crate::message::MessageSerializer<Self> {
         UInt64ValueSerializer
     }
@@ -171,6 +169,9 @@ impl crate::message::Message for UInt64Value {
 
 struct UInt64ValueSerializer;
 
+#[cfg_attr(not(feature = "_internal-semver"), doc(hidden))]
+impl crate::message::sealed::MessageSerializer for UInt64ValueSerializer {}
+
 impl crate::message::MessageSerializer<UInt64Value> for UInt64ValueSerializer {
     fn serialize_to_map(
         &self,
@@ -197,7 +198,6 @@ impl crate::message::Message for Int64Value {
         "type.googleapis.com/google.protobuf.Int64Value"
     }
 
-    #[allow(private_interfaces)]
     fn serializer() -> impl crate::message::MessageSerializer<Self>
     where
         Self: Sized + serde::ser::Serialize,
@@ -208,6 +208,9 @@ impl crate::message::Message for Int64Value {
 
 struct Int64ValueSerializer;
 
+#[cfg_attr(not(feature = "_internal-semver"), doc(hidden))]
+impl crate::message::sealed::MessageSerializer for Int64ValueSerializer {}
+
 impl crate::message::MessageSerializer<Int64Value> for Int64ValueSerializer {
     fn serialize_to_map(
         &self,
@@ -234,7 +237,6 @@ impl crate::message::Message for FloatValue {
         "type.googleapis.com/google.protobuf.FloatValue"
     }
 
-    #[allow(private_interfaces)]
     fn serializer() -> impl crate::message::MessageSerializer<Self>
     where
         Self: Sized + serde::ser::Serialize,
@@ -245,6 +247,9 @@ impl crate::message::Message for FloatValue {
 
 struct FloatValueSerializer;
 
+#[cfg_attr(not(feature = "_internal-semver"), doc(hidden))]
+impl crate::message::sealed::MessageSerializer for FloatValueSerializer {}
+
 impl crate::message::MessageSerializer<FloatValue> for FloatValueSerializer {
     fn serialize_to_map(
         &self,
@@ -271,7 +276,6 @@ impl crate::message::Message for DoubleValue {
         "type.googleapis.com/google.protobuf.DoubleValue"
     }
 
-    #[allow(private_interfaces)]
     fn serializer() -> impl crate::message::MessageSerializer<Self>
     where
         Self: Sized + serde::ser::Serialize,
@@ -282,6 +286,9 @@ impl crate::message::Message for DoubleValue {
 
 struct DoubleValueSerializer;
 
+#[cfg_attr(not(feature = "_internal-semver"), doc(hidden))]
+impl crate::message::sealed::MessageSerializer for DoubleValueSerializer {}
+
 impl crate::message::MessageSerializer<DoubleValue> for DoubleValueSerializer {
     fn serialize_to_map(
         &self,
@@ -308,7 +315,6 @@ impl crate::message::Message for BytesValue {
         "type.googleapis.com/google.protobuf.BytesValue"
     }
 
-    #[allow(private_interfaces)]
     fn serializer() -> impl crate::message::MessageSerializer<Self> {
         BytesValueSerializer
     }
@@ -316,6 +322,9 @@ impl crate::message::Message for BytesValue {
 
 struct BytesValueSerializer;
 
+#[cfg_attr(not(feature = "_internal-semver"), doc(hidden))]
+impl crate::message::sealed::MessageSerializer for BytesValueSerializer {}
+
 impl crate::message::MessageSerializer<BytesValue> for BytesValueSerializer {
     fn serialize_to_map(
         &self,
@@ -345,7 +354,7 @@ fn expected_string_value() -> crate::AnyError {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use super::*;
     use crate::Any;
     use crate::message::{Message, MessageSerializer};
@@ -376,7 +385,7 @@ mod test {
         let any = Any::from_msg(&input)?;
         let got = serde_json::to_value(&any)?;
         let want = serde_json::json!({
-            "@type": format!("type.googleapis.com/google.protobuf.{}", typename),
+            "@type": format!("type.googleapis.com/google.protobuf.{typename}"),
             "value": value,
         });
         assert_eq!(got, want);
@@ -461,7 +470,7 @@ mod test {
         T: Message + std::fmt::Debug + serde::ser::Serialize + serde::de::DeserializeOwned,
     {
         let map = serde_json::json!({
-            "@type": format!("type.googleapis.com/google.protobuf.{}", typename),
+            "@type": format!("type.googleapis.com/google.protobuf.{typename}"),
             "value": 0,
         });
 
@@ -469,7 +478,7 @@ mod test {
         let e = serializer.deserialize_from_map(map.as_object().unwrap());
 
         assert!(e.is_err());
-        let fmt = format!("{:?}", e);
+        let fmt = format!("{e:?}");
         assert!(fmt.contains("expected value field to be a string"), "{fmt}");
         Ok(())
     }
