@@ -1,3 +1,4 @@
+#![cfg_attr(docsrs, feature(doc_cfg))]
 /*!
 
 # Keyring
@@ -204,6 +205,7 @@ compile_error!("This crate cannot use both the sync and async versions of any cr
 // pick the *nix keystore
 //
 #[cfg(all(target_os = "linux", feature = "linux-native"))]
+#[cfg_attr(docsrs, doc(cfg(target_os = "linux")))]
 pub mod keyutils;
 #[cfg(all(
     target_os = "linux",
@@ -217,6 +219,10 @@ pub use keyutils as default;
     any(target_os = "linux", target_os = "freebsd", target_os = "openbsd"),
     any(feature = "sync-secret-service", feature = "async-secret-service"),
 ))]
+#[cfg_attr(
+    docsrs,
+    doc(cfg(any(target_os = "linux", target_os = "freebsd", target_os = "openbsd")))
+)]
 pub mod secret_service;
 #[cfg(all(
     any(target_os = "linux", target_os = "freebsd", target_os = "openbsd"),
@@ -235,6 +241,7 @@ pub use secret_service as default;
         feature = "linux-native-async-persistent",
     )
 ))]
+#[cfg_attr(docsrs, doc(cfg(target_os = "linux")))]
 pub mod keyutils_persistent;
 #[cfg(all(
     target_os = "linux",
@@ -265,6 +272,7 @@ pub use mock as default;
 // pick the Apple keystore
 //
 #[cfg(all(target_os = "macos", feature = "apple-native"))]
+#[cfg_attr(docsrs, doc(cfg(target_os = "macos")))]
 pub mod macos;
 #[cfg(all(target_os = "macos", feature = "apple-native"))]
 pub use macos as default;
@@ -272,6 +280,7 @@ pub use macos as default;
 pub use mock as default;
 
 #[cfg(all(target_os = "ios", feature = "apple-native"))]
+#[cfg_attr(docsrs, doc(cfg(target_os = "ios")))]
 pub mod ios;
 #[cfg(all(target_os = "ios", feature = "apple-native"))]
 pub use ios as default;
@@ -282,6 +291,7 @@ pub use mock as default;
 // pick the Windows keystore
 //
 #[cfg(all(target_os = "windows", feature = "windows-native"))]
+#[cfg_attr(docsrs, doc(cfg(target_os = "windows")))]
 pub mod windows;
 #[cfg(all(target_os = "windows", not(feature = "windows-native")))]
 pub use mock as default;
@@ -515,7 +525,6 @@ doc_comment::doctest!("../README.md", readme);
 #[allow(dead_code)]
 mod tests {
     use super::{credential::CredentialApi, Entry, Error, Result};
-    use rand::Rng;
     use std::collections::HashMap;
 
     /// Create a platform-specific credential given the constructor, service, and user
@@ -602,22 +611,21 @@ mod tests {
     /// the conflicts, and then do any needed cleanup once everything
     /// is working correctly.  So we export this function for tests to use.
     pub fn generate_random_string_of_len(len: usize) -> String {
-        // from the Rust Cookbook:
-        // https://rust-lang-nursery.github.io/rust-cookbook/algorithms/randomness.html
-        #[allow(unused_imports)]
-        use rand::Rng;
-        use rand::{distributions::Alphanumeric, thread_rng};
-        thread_rng()
-            .sample_iter(&Alphanumeric)
-            .take(len)
-            .map(char::from)
-            .collect()
+        use fastrand;
+        use std::iter::repeat_with;
+        repeat_with(fastrand::alphanumeric).take(len).collect()
     }
 
     pub fn generate_random_string() -> String {
         generate_random_string_of_len(30)
     }
 
+    fn generate_random_bytes_of_len(len: usize) -> Vec<u8> {
+        use fastrand;
+        use std::iter::repeat_with;
+        repeat_with(|| fastrand::u8(..)).take(len).collect()
+    }
+
     pub fn test_empty_service_and_user<F>(f: F)
     where
         F: Fn(&str, &str) -> Entry,
@@ -674,9 +682,8 @@ mod tests {
     {
         let name = generate_random_string();
         let entry = f(&name, &name);
-        let mut secret: [u8; 16] = [0; 16];
-        rand::rngs::OsRng.fill(&mut secret);
-        test_round_trip_secret("non-ascii password", &entry, &secret);
+        let secret = generate_random_bytes_of_len(24);
+        test_round_trip_secret("non-ascii password", &entry, secret.as_slice());
     }
 
     pub fn test_update<F>(f: F)
