@@ -1,20 +1,36 @@
-use crate::{Interner, VersionSetId, VersionSetUnionId};
-use itertools::Itertools;
 use std::fmt::Display;
 
+use itertools::Itertools;
+
+use crate::internal::id::ConditionId;
+use crate::{ConditionalRequirement, Interner, VersionSetId, VersionSetUnionId};
+
 /// Specifies the dependency of a solvable on a set of version sets.
 #[derive(Clone, Copy, Debug, Hash, Eq, PartialEq, Ord, PartialOrd)]
 #[cfg_attr(feature = "serde", derive(serde::Serialize, serde::Deserialize))]
+#[cfg_attr(feature = "serde", serde(rename_all = "snake_case"))]
 pub enum Requirement {
     /// Specifies a dependency on a single version set.
     Single(VersionSetId),
-    /// Specifies a dependency on the union (logical OR) of multiple version sets. A solvable
-    /// belonging to _any_ of the version sets contained in the union satisfies the requirement.
-    /// This variant is typically used for requirements that can be satisfied by two or more
-    /// version sets belonging to _different_ packages.
+    /// Specifies a dependency on the union (logical OR) of multiple version
+    /// sets. A solvable belonging to _any_ of the version sets contained in
+    /// the union satisfies the requirement. This variant is typically used
+    /// for requirements that can be satisfied by two or more version sets
+    /// belonging to _different_ packages.
     Union(VersionSetUnionId),
 }
 
+impl Requirement {
+    /// Constructs a `ConditionalRequirement` from this `Requirement` and a
+    /// condition.
+    pub fn with_condition(self, condition: ConditionId) -> ConditionalRequirement {
+        ConditionalRequirement {
+            condition: Some(condition),
+            requirement: self,
+        }
+    }
+}
+
 impl Default for Requirement {
     fn default() -> Self {
         Self::Single(Default::default())
@@ -65,24 +81,27 @@ pub(crate) struct DisplayRequirement<'i, I: Interner> {
 impl<I: Interner> Display for DisplayRequirement<'_, I> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match *self.requirement {
-            Requirement::Single(version_set) => write!(
-                f,
-                "{} {}",
-                self.interner
-                    .display_name(self.interner.version_set_name(version_set)),
-                self.interner.display_version_set(version_set)
-            ),
+            Requirement::Single(version_set) => {
+                let result = format!(
+                    "{} {}",
+                    self.interner
+                        .display_name(self.interner.version_set_name(version_set)),
+                    self.interner.display_version_set(version_set)
+                );
+                write!(f, "{}", result.trim_end())
+            }
             Requirement::Union(version_set_union) => {
                 let formatted_version_sets = self
                     .interner
                     .version_sets_in_union(version_set_union)
                     .format_with(" | ", |version_set, f| {
-                        f(&format_args!(
+                        let result = format!(
                             "{} {}",
                             self.interner
                                 .display_name(self.interner.version_set_name(version_set)),
                             self.interner.display_version_set(version_set)
-                        ))
+                        );
+                        f(&format_args!("{}", result.trim_end()))
                     });
 
                 write!(f, "{}", formatted_version_sets)
