@@ -31,16 +31,16 @@ pub trait comctl_shell_Himagelist: comctl_Himagelist {
 	///     1,
 	/// )?;
 	///
-	/// himgl.add_icons_from_shell(&["mp3", "wav"])?;
-	/// # w::HrResult::Ok(())
+	/// himgl.add_icon_from_shell(&["mp3", "wav"])?;
+	/// # Ok::<_, co::ERROR>(())
 	/// ```
-	fn add_icons_from_shell(&self,
+	fn add_icon_from_shell(&self,
 		file_extensions: &[impl AsRef<str>],
-	) -> HrResult<()>
+	) -> SysResult<()>
 	{
 		let sz = self.GetIconSize()?;
 		if sz != SIZE::new(16, 16) && sz != SIZE::new(32, 32) {
-			return Err(co::HRESULT::E_NOTIMPL); // only 16x16 or 32x32 icons can be loaded
+			return Err(co::ERROR::NOT_SUPPORTED); // only 16x16 or 32x32 icons can be loaded
 		}
 
 		for file_extension in file_extensions.iter() {
